#include <iostream>
#include <fstream>
#include <boost/format.hpp>
#include <boost/optional.hpp>
#include <boost/none.hpp>
#include <fileref.h>
#include <tstring.h>
#include <tag.h>
#include "abstract_tag.hpp"
#include "extract.hpp"
using namespace std;
using namespace boost;

namespace {
    optional<string> from_taglib_str(const TagLib::String& str) {
        if (str.isNull()) {
            return none;
        }
        else {
            return str.to8Bit(true); // true: gimme UTF-8
        }
    }
}

namespace TagMachine {

    void extract_tag(
        const std::string& music_fpath,
        const std::string& tag_fpath,
        const std::string& tag_format) {

        ofstream tag_stream(
            tag_fpath == "-" ? "/dev/fd/1" : tag_fpath.c_str(),
            ios::out);

        if (!tag_stream.good()) {
            throw (format("failed to write tag file: %s") % tag_fpath).str();
        }

        TagLib::FileRef music(music_fpath.c_str());
        if (music.isNull()) {
            throw (format("failed to manipulate music file: %s") % music_fpath).str();
        }

        AbstractTag tag;

        tag.title   = from_taglib_str(music.tag()->title  ());
        tag.artist  = from_taglib_str(music.tag()->artist ());
        tag.album   = from_taglib_str(music.tag()->album  ());
        tag.comment = from_taglib_str(music.tag()->comment());
        tag.genre   = from_taglib_str(music.tag()->genre  ());
        tag.year        = music.tag()->year()  == 0 ? none : optional<int         >(music.tag()->year ());
        tag.tracknumber = music.tag()->track() == 0 ? none : optional<unsigned int>(music.tag()->track());

        tag.write(tag_stream, force_find_format_by_name(tag_format));
    }
    
}
